package org.apache.onami.converters.net;

/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

import java.net.MalformedURLException;
import java.net.URL;

import org.apache.onami.converters.core.AbstractConverter;
import org.kohsuke.MetaInfServices;

import com.google.inject.Module;
import com.google.inject.TypeLiteral;

/**
 * Converter implementation for {@code java.net.URL}.
 */
@MetaInfServices( Module.class )
public final class URLConverter
    extends AbstractConverter<URL>
{

    /**
     * Pseudo URL prefix for loading from the class path: "classpath://"
     */
    public static final String CLASSPATH_URL_PREFIX = "classpath://";

    /**
     * {@inheritDoc}
     */
    public Object convert( String value, TypeLiteral<?> toType )
    {
        if ( value.startsWith( CLASSPATH_URL_PREFIX ) )
        {
            String path = value.substring( CLASSPATH_URL_PREFIX.length() );
            while ( '/' == path.charAt( 0 ) )
            {
                path = path.substring( 1 );
            }

            ClassLoader classLoader = null;
            try
            {
                classLoader = Thread.currentThread().getContextClassLoader();
            }
            catch ( SecurityException t )
            {
                // Cannot access thread context ClassLoader - falling back to system class loader...
            }
            if ( classLoader == null )
            {
                // No thread context class loader -> use class loader of this class.
                classLoader = URLConverter.class.getClassLoader();
            }
            URL url = classLoader.getResource( path );
            if ( url == null )
            {
                throw new IllegalArgumentException( "class path resource '"
                                              + path
                                              + "' cannot be resolved to URL because it does not exist" );
            }

            return url;
        }

        try
        {
            return new URL( value );
        }
        catch ( MalformedURLException e )
        {
            throw new IllegalArgumentException( "String value '" + value + "' is not a valid URL", e );
        }
    }

}
